package labone;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: zaic
 * Date: 3/21/12
 * Time: 6:55 PM
 */
public class DynamicImageDrawer {
    PlotPanel panel;
    BufferedImage image;
    boolean blend = false;
    boolean filter = false;

    DynamicImageDrawer(PlotPanel panel, BufferedImage image, boolean blend, boolean filter) {
        this.panel = panel;
        this.image = image;
        this.blend = blend;
        this.filter = filter;
    }

    void setBlend(boolean value) {
        blend = value;
    }

    void setFilter(boolean value) {
        filter = value;
    }

    /*
     *  rotate
     */
    
    Point2D.Double rotate(Point2D.Double p, double angle) {
        double s = Math.sin(angle);
        double c = Math.cos(angle);
        return new Point2D.Double(p.getX() * c - p.getY() * s, p.getX() * s + p.getY() * c);
    }
    
    //ArrayList<Point> getTrianglePoints(int triangle_size, int start_orientation, double angle) {
    ArrayList<Point> getTrianglePoints(int triangle_size, int start_orientation, double angle, int field[][]) {
        final int triangle_shift = 3 * triangle_size / 2;
        Point2D.Double corners[] = new Point2D.Double[3];
        corners[0] = new Point2D.Double(0, triangle_size);
        corners[1] = new Point2D.Double(triangle_size, 0);
        corners[2] = new Point2D.Double(start_orientation == 0 ? 0 : triangle_size, start_orientation == 0 ? 0 : triangle_size);
        for(int i = 0; i < 3; i++) {
            corners[i] = rotate(corners[i], angle);
            //System.err.println("x=" + corners[i].getX() + ", y=" + corners[i].getY());
        }

        //int field[][] = new int[4 * triangle_size][4 * triangle_size];
        for(int i = 0; i < field.length; i++) for(int j = 0; j < field[0].length; j++) field[i][j] = -1;
        for(int i = 0; i < 3; i++)
            GraphicsPrimitives.drawLine(field,
                    (int)corners[i].getX() + triangle_shift, (int)corners[i].getY() + triangle_shift,
                    (int)corners[(i + 1) % 3].getX() + triangle_shift, (int)corners[(i + 1) % 3].getY() + triangle_shift);
        
        double sum[] = new double[2];
        for(int i = 0; i < 3; i++) {
            sum[0] += corners[i].getX();
            sum[1] += corners[i].getY();
        }
        int mx = (int)(sum[0] /= 3.0) + triangle_shift;
        int my = (int)(sum[1] /= 3.0) + triangle_shift;

        ArrayList<Point> triangle_points = new ArrayList<Point>();
        field[my][mx] = 255;
        //triangle_points.add(new Point(mx - triangle_shift, my - triangle_shift));
        int dir[][] = { {0, 1}, {0, -1}, {1, 0}, {-1, 0} };
        for(int i = 0; i < triangle_points.size(); i++) {
            int curx = triangle_points.get(i).x + triangle_shift;
            int cury = triangle_points.get(i).y + triangle_shift;
            //System.err.println("curx = " + curx + ", cury = " + cury);
            if(curx == 0 || cury == 0 || curx == triangle_size * 3 - 1 || cury == triangle_size * 3 - 1) continue;
            for(int j = 0; j < 4; j++) {
                int nextx = curx + dir[j][0];
                int nexty = cury + dir[j][1];
                if(field[nextx][nexty] == -1) {
                    field[nextx][nexty] = 0;
                    triangle_points.add(new Point(nextx - triangle_shift, nexty - triangle_shift));
                }
            }
        }
        return triangle_points;
    }

    public void draw(Graphics2D graphics, double iteration) {
        int image_size = Math.min(panel.getHeight(), panel.getWidth()) / 3;
        int image_shift_width = (panel.getWidth() - image_size) / 2;
        int image_shift_height = (panel.getHeight() - image_size) / 2;

        int triangle_size = image_size / 4;
        int image_rgb[][] = new int[triangle_size][triangle_size];
        //ArrayList<Point> triangle_points = getTrianglePoints(triangle_size, 0, iteration * Math.PI * 2);
        int field[][] = new int[triangle_size * 3][triangle_size * 3];
        ArrayList<Point> triangle_points = getTrianglePoints(triangle_size, 0, iteration * Math.PI * 2, field);
        for(int i = 0; i < field.length; i++)
            for(int j = 0; j < field[i].length; j++) if(field[i][j] == 0) {
                GraphicsPrimitives.putPixel(graphics, image_shift_width + i, image_shift_height + j, Color.black);
            }

        /*
        for(Point i : triangle_points) {
            Point2D.Double real_point = rotate(new Point2D.Double(i.x, i.y), 0);
            int current_rgb = StaticImageDrawer.getImageRGB(image, (int)real_point.getX(), (int)real_point.getY());
            Color current_pixel_color = new Color(current_rgb);
            GraphicsPrimitives.putPixel(graphics, image_shift_width + i.x, image_shift_height + i.y, current_pixel_color);
        }
        */
    }
}
